require 'json'

module UU
  module OS
    module GVC
      # Table row class, it is used as return type in some {UU::OS::GVC::Table} methods.
      class TableRow

        # Flag if row is visible in form. Default is true. If is set to false row is completely hidden.
        #
        # @example
        #   # get table row
        #   table = view.get_components_by_code("table")[0]
        #   row = table.get_row(2)
        #
        #   # hide row
        #   row.visible = false
        #
        #   # make row visible again
        #   row.visible = true
        #
        # @return [Boolean]
        attr_accessor :visible

        # Table row code. Read only.
        #
        # @example
        #   # get table row
        #   table = view.get_components_by_code("table")[0]
        #   row = table.get_row(2)
        #
        #   # hide row
        #   code = row.code # table code as string "row3"
        #
        # @return [String]
        attr_reader :code
        
        # Creates new instance of TableRow.
        #
        # @param [String, Hash] data Initialization data in form of JSON string or Hash (optional)
        def initialize(data = nil)

          if data.kind_of?Hash
            @attributes = data
          elsif data.kind_of?String
            @attributes = JSON.parse(data, :symbolize_names => true)
          end
        end

        # Returns attributes representation as JSON String.
        #
        # @return [String] Attributes in JSON String.
        def to_json(options = {})
          attributes.to_json(options)
        end

        # Returns attributes representation as JSON String.
        #
        # @return [String] Attributes in JSON String.
        def to_s
          to_json
        end

        def visible
          attributes[:visible]
        end

        def visible=(visible)
          attributes[:visible] = visible
        end

        def code
          attributes[:code]
        end

        private

        # @private
        def attributes
          @attributes ||= {}
        end
      end

    end
  end
end