require 'uu/os/gvc/component'

module UU
  module OS
    module GVC
            
      #Uxmlfragment component. This component is meant for creating dynamic views by supplying fragments of UXML (can contain other gvc components, but is restrictive for other uxml elements, see documentation).
      #Documentation at: {https://plus4u.net/ues/sesm?SessFree=ues:UNI-BT:UES.DOC/EDITOR_V1/uu.os.gvc.uxmlfragment ues:UNI-BT:UES.DOC/EDITOR_V1/uu.os.gvc.uxmlfragment}
      class UxmlFragment < UU::OS::GVC::Component

        #Use this to set a fragment of UXML code (only String) you want to render. The UXML has to be valid and contain only allowed elements (see {https://plus4u.net/ues/sesm?SessFree=ues:UNI-BT:UES.DOC/EDITOR_V1/uu.os.gvc.uxmlfragment ues:UNI-BT:UES.DOC/EDITOR_V1/uu.os.gvc.uxmlfragment}).
        #The UXML doesnt need to have one root element containing everything - this the only exception to what should a well-formed XML look like. You can retrieve the UXML back using this accessor but
        #only within the scope of the same event in which the UXML was set. When handling another event in your controller, you wont be able to retrieve UXML from previous events, because for traffic-saving
        #reasons its not being sent as a part of the controller's viewmodel. If you wish to keep the state of your UXML between events it is advised not to entirely circumvent this limitation by manually
        #putting your UXML back into the viewmodel as a custom attribute (anti-pattern!), but rather to use a templating engine (erb and the like) to keep the bulk of the uxml as a template - a stateless
        #resource that then understandably doesnt need to be a part of the viewmodel and keep only the necessary pieces of data, that define the state and populate the template, in the viewmodel. 
        #
        #@example
        #   # Basic usage example:
        #   # Construct an UXML snippet with hello world and informing about current time in bold text
        #   uxml = "<core:text>Hello world! </core:text><core:text bold=\"true\">Current time is: #{Time.now}</core:text>"
        #
        #   # Set the UXML to a fragment component
        #   fragment_component = view.get_components_by_code('fragment')[0]
        #   fragment_component.data = uxml #Done! Now you should see your UXML hello world text rendered in the browser as HTML.
        attr_accessor :data

        #Attribute containing uri of the fragment component's datasource (should be valid UXML).
        #It can be set only by editing the containing sheet's content.
        attr_reader :data_uri
        
        def data
          #use the special attribute to present the user with the original uxml string he set
          attributes[:data].uxml unless (attributes[:data].nil? || !(attributes[:data].kind_of?UxmlData))
        end

        #this not your ordinary setter: it transforms the uxml to html using remote service
        def data=(indata)
          
          #there's no need to call a remote service for this
          return indata if indata.nil? or indata == ""

          uxmlData=nil
          if indata.kind_of?UxmlData
            #input data are already of the special type, save it without doing anything
            uxmlData=indata
          elsif indata.kind_of?String
            #assume input UXML and initialize the special data type with html returned by the remote service
            begin
              uu_uri = nil
              if @container
                if @container.instance_variable_get("@ctx")
                  if @container.instance_variable_get("@ctx").uu_uri
                    uu_uri = @container.instance_variable_get("@ctx").uu_uri.to_s
                  end
                elsif @container.instance_variable_get("@context")
                  if @container.instance_variable_get("@context").main_entity_uri
                    uu_uri = @container.instance_variable_get("@context").main_entity_uri.to_s
                  end
                end
              end

              uxmlData=UxmlData.new(UxmlClient.transform(indata,uu_uri ))
              #save the original uxml to a special attribute from where it can be retrieved later
              uxmlData.uxml=indata
            end
          else
            raise ArgumentError.new('Unsupported data format, must be String')
          end
          attributes[:data] = uxmlData
        end

        def data_uri
          attributes[:data_uri]
        end

        #@private
        def initialize(container, data = nil)
          super(container, data)
          #snakecase vs camelCase - TODO sort out the problem with both of these attributes being sent
          attributes[:data_uri] = data[:dataUri]
        end

      end

      #@private
      #special data type that is meant to mainly present the transformed html, but also keeps the original uxml
      #so the user can retrieve it with data getter after setting it
        class UxmlData < String
          @uxml =''
          def uxml=(inuxml)
            @uxml=inuxml
          end

          def uxml
            @uxml
          end
        end

      #@private
      #rest client for communication with uxmlfragment service
      class UxmlClient
        PATH = 'uu/os/gvc/uxmlfragment'
        def self.transform(indata, uri=nil)
          if (defined?(UU::OS::REST))
            svc = UU::OS::REST::RemoteClient.new(UxmlClient)
            svc.add_header("Accept","text/html")
            svc.add_header("Content-type","text/xml")
            return svc.post('transform', uri, indata)
          else
            return nil
          end
        end
      end
    end
  end
end
